/***************************************************************************
    
    Handle preview mode
    
    It is displayed in 3 layers
    
    
    Engine : Call setPreviewMode and amdPreview depending on the actions
            previewMode is the **current** preview mode
    
    admPreview : 
          Allocate/desallocate ressources
          Build preview window
          Call display properly to display it
          
    GUI_PreviewXXXX
          UI toolkit to actually display the window
          See GUI_ui.h to see them
                 
    
    
    copyright            : (C) 2007 by mean
    email                : fixounet@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ADM_assert.h>
#include <ADM_assert.h>
#include "config.h"

#include "default.h"

#include "ADM_toolkit/toolkit.hxx"
#include "ADM_editor/ADM_edit.hxx"
#include "ADM_video/ADM_genvideo.hxx"
#include "ADM_filter/video_filters.h"


#include "GUI_render.h"

#include "ADM_osSupport/ADM_debugID.h"
#define MODULE_NAME MODULE_PREVIEW
#include "ADM_osSupport/ADM_debug.h"
#include "ADM_libraries/ADM_libswscale/ADM_mp.h"

#include "GUI_ui.h"
#define MAX(a,b) ( (a)>(b) ? (a) : (b) )

extern FILTER  videofilters[MAX_FILTER];
extern uint32_t nb_active_filter;

static void previewBlit(ADMImage *from,ADMImage *to,uint32_t startx,uint32_t starty);

static   AVDMGenericVideoStream *preview=NULL;
static ADMImage *previewImage;
static ADM_PREVIEW_MODE previewMode=ADM_PREVIEW_NONE;
static uint32_t _mainW=0,_mainH=0;
static uint32_t _displayW=0,_displayH=0;
static uint32_t defered_display=0;
static renderZoom zoom=ZOOM_1_1;
static ADMImage             *resized=NULL;
static ADM_MplayerResize    *resizer=NULL;
static ADMImage             *original=NULL;
static uint32_t _resizedW=0,_resizedH=0;
/**
      \fn getPreviewMode
      \brief returns current preview mode
      @return current preview mode

*/

ADM_PREVIEW_MODE getPreviewMode(void)
{
  return previewMode; 
}

void changePreviewZoom(renderZoom nzoom)
{
    admPreview::stop();
    zoom=nzoom;
    admPreview::start();
  
}

/**
      \fn setPreviewMode
      \brief set current preview mode an update UI
      @param mode  new preview mode

*/

 void setPreviewMode(ADM_PREVIEW_MODE mode)
{
  previewMode=mode; 
  UI_setCurrentPreview( (int)mode);
}
/**
      \fn deferDisplay
      \brief Enable or disable defered display

*/
void admPreview::deferDisplay(uint32_t onoff)
{
      
      if(onoff)
      {
        renderStartPlaying();
        defered_display=1;
      }
      else
      {
        renderStopPlaying();
        defered_display=0;
      }
  
}
/**
      \fn admPreview::start
      \brief start preview

*/

void 	admPreview::start( void )
{
            aprintf("--killing\n");
            getFirstVideoFilter();
            
            preview=videofilters[  nb_active_filter-1].filter;
            aprintf("--spawning\n");
            ADM_assert(!previewImage)
                
            previewImage=new ADMImage(preview->getInfo()->width,preview->getInfo()->height);
            
            ADM_assert(!original);
            switch(previewMode)
            {
              case  ADM_PREVIEW_SEPARATE:
                  GUI_PreviewInit(preview->getInfo()->width,preview->getInfo()->height,0);
                  
                  /* no break here, not a mistake */
              case  ADM_PREVIEW_NONE:
              
                  _displayW=_mainW;
                  _displayH=_mainH;
                  break;
              case  ADM_PREVIEW_OUTPUT:
                  _displayW=preview->getInfo()->width;
                  _displayH=preview->getInfo()->height;
                  break;
              case  ADM_PREVIEW_SIDE:
              {
                  
                  _displayH=MAX(_mainH,preview->getInfo()->height);
                  _displayW=_mainW+preview->getInfo()->width;
                  original=new ADMImage(_displayW,_displayH);
                  break;
              }
              case  ADM_PREVIEW_TOP:
              {
                  
                  _displayW=MAX(_mainW,preview->getInfo()->width);
                  _displayH=_mainH+preview->getInfo()->height;
                  original=new ADMImage(_displayW,_displayH);
                  break;
              }
              default: ADM_assert(0);
            }
        if(zoom!=ZOOM_1_1)
        {
            int mul;
            switch(zoom)
            {
                    case ZOOM_1_4: mul=1;break;
                    case ZOOM_1_2: mul=2;break;
                    case ZOOM_1_1: mul=4;break;
                    case ZOOM_2:   mul=8;break;
                    case ZOOM_4:   mul=16;break;
                    default : ADM_assert(0);
    
            }
            _resizedW=(_displayW*mul+3)/4;
            _resizedH=(_displayH*mul+3)/4;
    
            if(_resizedW&1) _resizedW++;
            if(_resizedH&1) _resizedH++;
            ADM_assert(!resized);
            resized=new ADMImage(_resizedW,_resizedH);
            ADM_assert(!resizer);
            resizer=new  ADM_MplayerResize(_displayW,_displayH,_resizedW,_resizedH);
            renderResize(_resizedW,_resizedH);
        }else
        {
             renderResize(_displayW,_displayH);
        }
}
/**
      \fn admPreview::stop
      \brief kill preview window and associated datas
*/

void admPreview::stop( void )
{
      if(previewMode==ADM_PREVIEW_SEPARATE)
                GUI_PreviewEnd();
      if(  previewMode==ADM_PREVIEW_SIDE || previewMode==ADM_PREVIEW_TOP)
      {
        ADM_assert(original);
        original=NULL; 
      }
      renderResize(_mainW,_mainH);
      if(previewImage)
      {
        delete  previewImage; 
        previewImage=NULL;
      }
      if(zoom!=ZOOM_1_1)
      {
          ADM_assert(resized);
          ADM_assert(resizer);
          delete resized;
          delete resizer;
          resized=NULL;
          resizer=NULL;
      }
}
/**
      \fn admPreview::updateFilters
      \brief Signal from filter that the filter chain has been updated
      @param w : width
      @param h : height
*/

void admPreview::updateFilters(AVDMGenericVideoStream *first,AVDMGenericVideoStream *last)
{
   switch(previewMode)
            {
              case  ADM_PREVIEW_SEPARATE:
                  preview=last;
                  break;
                  /* no break here, not a mistake */
              case  ADM_PREVIEW_NONE:
                  break;
              case  ADM_PREVIEW_OUTPUT:
                preview=last;
                break;
              case  ADM_PREVIEW_SIDE:
              {
                  
                  preview=last;
                  break;
              }
              case  ADM_PREVIEW_TOP:
              {
                  
                  preview=last;
                  break;
              }
              default: ADM_assert(0);
            }
  
}

/**
      \fn admPreview::setMainDimension
      \brief Update width & height of input video
      @param w : width
      @param h : height
*/

void admPreview::setMainDimension(uint32_t w, uint32_t h)
{
  _mainW=w;
  _mainH=h;
  _displayW=w;
  _displayH=h;
  renderResize(w,h);
  
}
/**
      \fn admPreview::update
      \brief display data associated with framenum image
      @param image : current main image (input)
      @param framenum, framenumber
*/

void admPreview::update(uint32_t framenum,ADMImage *image)
{
    uint32_t fl,len;	

    switch(previewMode)
    {
      case ADM_PREVIEW_NONE:
        if(image)
        {
            if(zoom==ZOOM_1_1 )
            {
               if(!defered_display) 
                  renderUpdateImage(image->data);
            }else
            {
                ADM_assert(resizer);
                ADM_assert(resized);
                resizer->resize(image->data,resized->data);
                if(!defered_display) 
                  renderUpdateImage(resized->data);
            }
        }
        break;
      case ADM_PREVIEW_OUTPUT:
            if(framenum<=preview->getInfo()->nb_frames-1)
                  {
                          if(!preview->getFrameNumberNoAlloc(framenum,&len,previewImage,&fl)) return;
                          if(zoom==ZOOM_1_1)
                          {
                            if(!defered_display) renderUpdateImage(previewImage->data);
                          }
                          else
                          {
                             resizer->resize(previewImage->data,resized->data);
                             if(!defered_display) 
                                  renderUpdateImage(resized->data);
                          }
                  }
            break;
      case ADM_PREVIEW_SEPARATE:
            ADM_assert(preview);
            ADM_assert(previewImage);
            if(zoom==ZOOM_1_1)
            {
              if(image && !defered_display) renderUpdateImage(image->data);
            }else
            {
                ADM_assert(resizer);
                ADM_assert(resized);
                resizer->resize(image->data,resized->data);
                if(!defered_display) 
                  renderUpdateImage(resized->data);
            }
          if( GUI_PreviewStillAlive())
          {
                  aprintf("Preview: Ask for frame %lu\n",framenum);
                  if(framenum<=preview->getInfo()->nb_frames-1)
                  {
                          preview->getFrameNumberNoAlloc(framenum,&len,previewImage,&fl);
                          if(!defered_display) GUI_PreviewUpdate(previewImage->data);
                  }
          }
          break;
      case ADM_PREVIEW_SIDE:
              ADM_assert(preview);
              ADM_assert(previewImage);
              ADM_assert(original);
              
              
              previewBlit(image,original,0,0);
              if(preview->getFrameNumberNoAlloc(framenum,&len,previewImage,&fl))
              {
                previewBlit(previewImage,original,_mainW,0);
              }
              if(zoom==ZOOM_1_1)
              {
                if(!defered_display)
                  renderUpdateImage(original->data);
              }else
              {
                resizer->resize(original->data,resized->data);
                if(!defered_display) 
                  renderUpdateImage(resized->data);
              }
              break;
        
      case ADM_PREVIEW_TOP:
              ADM_assert(preview);
              ADM_assert(previewImage);
              ADM_assert(original);
              
              previewBlit(image,original,0,0);
              if(preview->getFrameNumberNoAlloc(framenum,&len,previewImage,&fl))
              {
                previewBlit(previewImage,original,0,_mainH);
              }
              if(zoom==ZOOM_1_1)
              {
                if(!defered_display)
                  renderUpdateImage(original->data);
              }else
              {
                resizer->resize(original->data,resized->data);
                if(!defered_display) 
                  renderUpdateImage(resized->data);
              }
              break;
      default: ADM_assert(0);
    }
}
/**
      \fn previewBlit(ADMImage *from,ADMImage *to,uint32_t startx,uint32_t starty)
      \brief Blit "from" to "to" at position startx,starty
*/

void previewBlit(ADMImage *from,ADMImage *to,uint32_t startx,uint32_t starty)
{
  
  from->copyTo(to,startx,starty);
  
}
/**
      \fn displayNow
      \brief display on screen immediately
*/

void admPreview::displayNow(uint32_t framenum,ADMImage *image)
{
    uint32_t fl,len;	

    switch(previewMode)
    {
      case ADM_PREVIEW_NONE:
           
           if(zoom==ZOOM_1_1 )
            {
                renderUpdateImage(image->data);
            }else
            {
                ADM_assert(resized);
                renderUpdateImage(resized->data);
            }
        break;
      case ADM_PREVIEW_OUTPUT:
            if(zoom==ZOOM_1_1 )
            {
                renderUpdateImage(previewImage->data);
            }else
            {
                ADM_assert(resized);
                renderUpdateImage(resized->data);
            }
            break;
      case ADM_PREVIEW_SEPARATE:
            if(zoom==ZOOM_1_1 )
            {
                renderUpdateImage(image->data);
            }else{
                ADM_assert(resized);
                renderUpdateImage(resized->data);
            }
            if( GUI_PreviewStillAlive())
            {
                GUI_PreviewUpdate(previewImage->data);
            }
          break;
      case ADM_PREVIEW_SIDE:
      case ADM_PREVIEW_TOP:
            if(zoom==ZOOM_1_1)
            {
               renderUpdateImage(original->data);
            }else
            {
              ADM_assert(resized);
              renderUpdateImage(resized->data);
            }
              break;
      default: ADM_assert(0);
    }
}
